/*******************************************************************************
 * (c) Copyright 2011 Actel Corporation.  All rights reserved.
 * 
 * Public function declarations, register and bit level definitions for 
 * National's DP83848 PHY interface.
 * This file contains bit definitions of only DP83848 specific registers. 
 * For bit definitions of other (standard MII) registers, refer phy.h.
 * 
 * 
 * SVN $Revision: 3818 $
 * SVN $Date: 2011-09-09 17:31:01 +0530 (Fri, 09 Sep 2011) $
 */
#ifndef DP83848_PHY_H_
#define DP83848_PHY_H_

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------
 * DP83848 PHY address
 */
#define DP83848_PHY_ADDR                   0x01

/*------------------------------------------------------------------------------
 * DP83848 PHY specific register offsets and bit definitions
 */
#define DP83848_PHY_STATUS                 0x10u
#define DP83848_PHY_STATUS_FULLDUPLEX	   0x0004u
#define DP83848_PHY_STATUS_LINK	           0x0001u
#define DP83848_PHY_STATUS_100             0x0000u
#define DP83848_PHY_STATUS_10              0x0002u
/**************************************************************************/
/* Public Function declarations                                           */
/**************************************************************************/

/***************************************************************************//**
  The function dp83848_init()is used to initialize National's DP83848 PHY 
  interface. This function is called from MSS_MAC_init() function when
  DP83848 PHY is the selected as PHY interface. PHY interface callback 
  mechanism defined in PHY_devices of phy.c is used to make this funciton call. 
  This function resets the DP83848 interface, disables loopback, enables 
  auto-negotiation and restarts the same when auto-negotiation is enabled in 
  the MAC configuration record. 
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer.
 
  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"

    PHY_device[cfg->phy].init( cfg );
 */
return_t 
dp83848_init
(
    MAC_cfg_t * cfg
);

/***************************************************************************//**
  The function dp83848_config_aneg() is used to configure and perform auto
  negotiation for DP83848 PHY interface. This function is called from MSS_MAC_init()
  using PHY_device callback mechanism when the PHY interface is chosen as 
  DP83848 by the user. This functions advertises the capabilities configured by 
  the user after complying with the DP83848 capabilities. Then it restarts auto
  negotiation and waits till it is completed. This function internally polls the 
  "auto negotiation complete" flag of DP83848 status register and returns once
  the flag indicates that auto negotiation is completed or timed out.
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer.

  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"

    PHY_device[cfg->phy].config_aneg( cfg );
  @endcode
 */
return_t
dp83848_config_aneg
(
    MAC_cfg_t * cfg
);

/***************************************************************************//**
  The function dp83848_read_status() is used to read the status of the DP83848
  PHY once initiated auto-negoation is completed. This function uses the cfg
  parameter as out param. This function reads the link partner ability register
  of the PHY to find the negotiated abilities. Accordingly some of the elements
  of configuration record will be modified, which will be used in initialization 
  of Ethernet MAC. 
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer. Some elements of this record will be modified
    after successful execution of this function. 

  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"
            
    PHY_device[cfg->phy].read_status( cfg );
  @endcode
 */
return_t 
dp83848_read_status
(
    MAC_cfg_t * cfg
);


#ifdef __cplusplus
}
#endif

#endif /* DP83848_PHY_H_ */


