/*******************************************************************************
 * (c) Copyright 2011 Actel Corporation.  All rights reserved.
 *
 * This source file contains PHY (MDD) management callbacks for various 
 * PHYs supported. 
 *
 * SVN $Revision: 4969 $
 * SVN $Date: 2013-01-11 21:50:58 +0000 (Fri, 11 Jan 2013) $
 */

#include <stdio.h>
#include <stddef.h>
#include "phy.h"
#include "mss_ethernet_sgmii_phy.h"
#include "m88e1111_phy.h"
#include "m88e1340_phy.h"
#include "dp83848_phy.h"
#include "mss_ethernet_mac.h"


#ifdef __cplusplus
extern "C" {
#endif

/**************************************************************************/
/* PHY device driver definitions                                          */
/**************************************************************************/
const PHY_dev_t PHY_device[LAST_PHY] = { 
                                    {
                                        /* SGMII ID */
                                        0x00000000,
                                        /* Default ID Mask */
                                        0xFFFFFFFF,
                                        /* PHY name */
                                        (const uint8_t  *)"Internal SGMII",
                                        /* Internal SGMII address */
                                        SGMII_PHY_ADDR,
                                        /* Feature supported: For advertising */
                                        (SUPPORTED_10baseT_Half | SUPPORTED_10baseT_Full | 
                                         SUPPORTED_100baseT_Half | SUPPORTED_100baseT_Full |
                                         SUPPORTED_1000baseT_Half | SUPPORTED_1000baseT_Full),
                                        /* PHY init callback */
                                        sgmii_init,
                                        /* PHY discover (auto-detect) callback */
                                        NULL,
                                        /* PHY auto-negotiation configuration callback */
                                        sgmii_config_aneg,
                                        /* PHY read status callback */
                                        sgmii_read_status
                                    },
                                    {
                                        /* M88E1111 PHY ID */
                                        0x01410cc0,
                                        /* M88E1111 ID Mask */
                                        0xfffffff0,
                                        /* PHY name */
                                        (const uint8_t  *)"Marvell 88E1111",
                                        /* PHY address */
                                        M88E1111_PHY_ADDR,
                                        /* Features supported: For advertising */
                                        (SUPPORTED_10baseT_Half | SUPPORTED_10baseT_Full | 
                                         SUPPORTED_100baseT_Half | SUPPORTED_100baseT_Full |
                                         SUPPORTED_1000baseT_Half | SUPPORTED_1000baseT_Full |
                                         SUPPORTED_Autoneg | SUPPORTED_MII | SUPPORTED_FIBRE |
                                         SUPPORTED_Pause | SUPPORTED_Asym_Pause),
                                        /* PHY init callback */
                                        m88e1111_init,
                                        /* PHY discover (auto-detect) callback */
                                        NULL,
                                        /* PHY auto-negotiation configuration callback */
                                        m88e1111_config_aneg,
                                        /* PHY read status callback */
                                        m88e1111_read_status
                                    },
                                    {
                                        /* DP83848 PHY ID */
                                        0x080017A2,
                                        /* DP83848 ID Mask */
                                        0xfffffff0,
                                        /* PHY name */
                                        (const uint8_t  *)"National DP83848",
                                        /* PHY address */
                                        DP83848_PHY_ADDR,
                                        /* Features supported: For advertising */
                                        (SUPPORTED_10baseT_Half | SUPPORTED_10baseT_Full | 
                                         SUPPORTED_100baseT_Half | SUPPORTED_100baseT_Full |
                                         SUPPORTED_Autoneg | SUPPORTED_MII |
                                         SUPPORTED_Pause | SUPPORTED_Asym_Pause),
                                        /* PHY init callback */
                                        dp83848_init,
                                        /* PHY discover (auto-detect) callback */
                                        NULL,
                                        /* PHY auto-negotiation configuration callback */
                                        dp83848_config_aneg,
                                        /* PHY read status callback */
                                        dp83848_read_status
                                    },
                                    {
                                        /* M88E1340 PHY ID */
                                        0x01410DC0,
                                        /* M88E1340 ID Mask */
                                        0xfffffff0,
                                        /* PHY name */
                                        (const uint8_t  *)"Marvell 88E1340",
                                        /* PHY address */
                                        M88E1340_PHY_ADDR,
                                        /* Features supported: For advertising */
                                        (SUPPORTED_10baseT_Half | SUPPORTED_10baseT_Full | 
                                         SUPPORTED_100baseT_Half | SUPPORTED_100baseT_Full |
                                         SUPPORTED_1000baseT_Half | SUPPORTED_1000baseT_Full |
                                         SUPPORTED_Autoneg | SUPPORTED_MII | SUPPORTED_FIBRE |
                                         SUPPORTED_Pause | SUPPORTED_Asym_Pause),
                                        /* PHY init callback */
                                        m88e1340_init,
                                        /* PHY discover (auto-detect) callback */
                                        NULL,
                                        /* PHY auto-negotiation configuration callback */
                                        m88e1340_config_aneg,
                                        /* PHY read status callback */
                                        m88e1340_read_status
                                    }
                                };

/**************************************************************************/
/* Private function definitions                                          */
/**************************************************************************/

/**************************************************************************//**
 * This function is used to advertise the Ethernet PHY capabilities while 
 * kick starting the auto-negotiation process. This function verifies PHY 
 * capabilities supported w.r.to the ones asked to be advertised. 
 * This function returns 1 on successful execution, returns 0 otherwise.
 */
static return_t 
phy_config_advert
(
    uint8_t phyaddr, 
    MAC_cfg_t * cfg, 
    uint32_t supported
)
{
    uint32_t advertise = 0;
    uint32_t oldadv, adv;
    uint32_t err, changed = 0;

    /* Only allow advertising what
     * this PHY supports */
    if( ( cfg->speed == MAC10MBPS) && 
        (cfg->fullduplex == MSS_MAC_FULLDUPLEX_DISABLE) )
        advertise |= ADVERTISED_10baseT_Half;
    if( ( cfg->speed == MAC10MBPS) && 
        (cfg->fullduplex == MSS_MAC_FULLDUPLEX_ENABLE) )
        advertise |= ADVERTISED_10baseT_Full;    
    if( ( cfg->speed == MAC100MBPS) && 
        (cfg->fullduplex == MSS_MAC_FULLDUPLEX_DISABLE) )
        advertise |= ADVERTISED_100baseT_Half;
    if( ( cfg->speed == MAC100MBPS) && 
        (cfg->fullduplex == MSS_MAC_FULLDUPLEX_ENABLE) )
        advertise |= ADVERTISED_100baseT_Full;
    if( ( cfg->speed == MAC1000MBPS) && 
        (cfg->fullduplex == MSS_MAC_FULLDUPLEX_DISABLE) )
        advertise |= ADVERTISED_1000baseT_Half;
    if( ( cfg->speed == MAC1000MBPS) && 
        (cfg->fullduplex == MSS_MAC_FULLDUPLEX_ENABLE) )
        advertise |= ADVERTISED_1000baseT_Full;
    
    /* Advertise only what are supported */
    advertise &= supported;
    /* Setup standard advertisement */
    oldadv = adv = MSS_MAC_read_phy_reg(phyaddr,
                                        MII_ADVERTISE);

    adv &= ~(ADVERTISE_ALL | ADVERTISE_100BASE4 | 
             ADVERTISE_PAUSE_CAP | ADVERTISE_PAUSE_ASYM);

    if (advertise & ADVERTISED_10baseT_Half)
            adv |= ADVERTISE_10HALF;
    if (advertise & ADVERTISED_10baseT_Full)
            adv |= ADVERTISE_10FULL;
    if (advertise & ADVERTISED_100baseT_Half)
            adv |= ADVERTISE_100HALF;
    if (advertise & ADVERTISED_100baseT_Full)
            adv |= ADVERTISE_100FULL;
    if (advertise & ADVERTISED_Pause)
            adv |= ADVERTISE_PAUSE_CAP;
    if (advertise & ADVERTISED_Asym_Pause)
            adv |= ADVERTISE_PAUSE_ASYM;

    if (adv != oldadv) 
    {
        err = MSS_MAC_write_phy_reg(phyaddr,
                                   MII_ADVERTISE, adv);
        if (err == 0)
                return err;
        changed = 1;
    }

    /* Configure gigabit if it's supported */
    if (supported & (SUPPORTED_1000baseT_Half |
                            SUPPORTED_1000baseT_Full)) 
    {
        oldadv = adv = MSS_MAC_read_phy_reg(phyaddr,
                                            MII_CTRL1000);

        adv &= ~(ADVERTISE_1000FULL | ADVERTISE_1000HALF);
        if (advertise & SUPPORTED_1000baseT_Half)
                adv |= ADVERTISE_1000HALF;
        if (advertise & SUPPORTED_1000baseT_Full)
                adv |= ADVERTISE_1000FULL;

        if (adv != oldadv) 
        {
            err = MSS_MAC_write_phy_reg(phyaddr, 
                                        MII_CTRL1000, adv);
            if (err == 0)
                    return err;
            changed = 1;
        }
    }

    return changed;
}

/**************************************************************************//**
 * This function is used to restart the auto-negotiation process of the
 * Ethernet PHY indicated by the argument phyaddr. This function returns 1
 * on successful execution, returns 0 otherwise.
 */
static uint32_t phy_restart_aneg(uint8_t phyaddr)
{
        uint32_t ctl;
        /* This timeout should be ideally >= 256ms.
           becuase, while operating at 1000Mbps
           auto-negotiation needs four Link Control Words (LCW).
           Each LCW has four Fast Link Pulses(FLPs). Time between
           two FLPs is 16ms. 
           Hence time for four FLPs = 4 * 16 = 64ms.
           Time for four LCWs = 4* 64= 256ms:
           But worst case scenario  */
        uint32_t timeout = 900000;

        ctl = MSS_MAC_read_phy_reg(phyaddr, 
                                   MII_BMCR);
        if (ctl == 0)
            return ctl;

        ctl |= (BMCR_ANENABLE | BMCR_ANRESTART);

        /* Don't isolate the PHY if we're negotiating */
        ctl &= ~(BMCR_ISOLATE);

        ctl = MSS_MAC_write_phy_reg(phyaddr, 
                                    MII_BMCR, ctl);
        /*Delay for completing auto-neg */
        while(timeout--);
        timeout = 1000000;
        /* wait here till auto-negotiation is complted or timedout */
        do
        {
            ctl = MSS_MAC_read_phy_reg(phyaddr, MII_BMSR);
            timeout--;
        }while( timeout && !(ctl & BMSR_ANEGCOMPLETE) );

        return ctl;
}

/**************************************************************************/
/* Public function definitions                                            */
/**************************************************************************/

/**************************************************************************//**
 * See phy.h for details of how to use this function.
 */
return_t phy_setup_forced(PHY_type_t phy, MAC_cfg_t * cfg)
{
    uint32_t err;
    uint32_t ctl = 0;

    if (MAC1000MBPS == cfg->speed)
            ctl |= BMCR_SPEED1000;
    else if (MAC100MBPS == cfg->speed)
            ctl |= BMCR_SPEED100;

    if (MSS_MAC_FULLDUPLEX_ENABLE == cfg->fullduplex)
            ctl |= BMCR_FULLDPLX;
    
    err = MSS_MAC_write_phy_reg(PHY_device[phy].addr,
                                MII_BMCR, ctl);

    return err;
}

/**************************************************************************//**
 * See phy.h for details of how to use this function.
 */
uint32_t phy_config_aneg(PHY_type_t phy, MAC_cfg_t * cfg)
{
    uint32_t result = 0u;

    if (MSS_MAC_AUTO_NEGOTIATION_ENABLE != cfg->auto_negotiation)
            return phy_setup_forced(phy, cfg);

    result = phy_config_advert(PHY_device[phy].addr, cfg, 
                               PHY_device[phy].supported);
    if (result == 0) 
    {
        /* Advertisment hasn't changed, but maybe aneg was never on to
         * begin with?  Or maybe phy was isolated? */
        uint32_t ctl = MSS_MAC_read_phy_reg(PHY_device[phy].addr,
                                            MII_BMCR);
        if (ctl == 0)
            return ctl;
        if (!(ctl & BMCR_ANENABLE) || (ctl & BMCR_ISOLATE))
                result = 1; /* do restart aneg */
    }

    /* Only restart aneg if we are advertising something different
     * than we were before.  */
    if (result > 0)
            result = phy_restart_aneg(PHY_device[phy].addr);
    return result;
}
/**************************************************************************//**
 * See phy.h for details of how to use this function.
 */
return_t phy_update_link(PHY_type_t phy)
{
    uint32_t status;

    /* Read link and autonegotiation status */
    status = MSS_MAC_read_phy_reg(PHY_device[phy].addr,
                                  MII_BMSR);
    if (status == 0)
            return status;
    if ((status & BMSR_LSTATUS) == 0)
        status = 0; /* link is down */
    else
        status = 1; /* link is up */
    return status;
}


#ifdef __cplusplus
}
#endif

/******************************** END OF FILE ******************************/






